part of '../fl_camera.dart';

class FlCamera extends StatefulWidget {
  const FlCamera(
      {Key? key,
      this.overlay,
      this.uninitialized,
      this.onFlashChange,
      this.isFullScreen = true})
      : super(key: key);

  /// 显示在预览框上面
  final Widget? overlay;

  /// 相机在未初始化时显示的布局
  final Widget? uninitialized;

  /// 闪光灯变化
  final ValueChanged<FlashState>? onFlashChange;

  /// 是否全屏
  final bool isFullScreen;

  @override
  _FlCameraStateWidget createState() => _FlCameraStateWidget();
}

class _FlCameraStateWidget extends FlCameraState<FlCamera> {
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addPostFrameCallback((Duration time) => init());
  }

  Future<void> init() async {
    fullScreen = widget.isFullScreen;

    /// 添加消息回调
    await initEvent(eventListen);

    /// 初始化相机
    if (await initCamera()) setState(() {});
  }

  @override
  void onFlashChange(FlashState state) {
    super.onFlashChange(state);
    if (widget.onFlashChange != null) widget.onFlashChange!(state);
  }

  void eventListen(dynamic data) {
    print('收到原生发送来的消息$data');
  }

  @override
  void didUpdateWidget(covariant FlCamera oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.overlay != widget.overlay ||
        oldWidget.isFullScreen != widget.isFullScreen ||
        oldWidget.onFlashChange != widget.onFlashChange ||
        oldWidget.uninitialized != widget.uninitialized) {
      cameraMethodCall.dispose().then((bool value) {
        if (value) init();
      });
    }
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      init();
    } else {
      super.didChangeAppLifecycleState(state);
    }
  }

  @override
  Widget build(BuildContext context) {
    Widget camera = super.build(context);
    if (widget.overlay != null)
      camera = Stack(children: <Widget>[
        camera,
        SizedBox.expand(child: widget.overlay),
      ]);
    return camera;
  }
}

abstract class FlCameraState<T extends StatefulWidget> extends State<T>
    with WidgetsBindingObserver {
  /// 相机配置信息
  FlCameraOptions? cameraOptions;

  /// 相机消息通道
  late FLCameraEvent cameraEven;

  /// 相机方法
  late FlCameraMethodCall cameraMethodCall;

  /// 当前使用的channel
  MethodChannel? currentChannel;

  /// 相机在未初始化时显示的布局
  Widget? uninitialized;

  /// 是否全屏 全屏会拉伸相机画面
  bool fullScreen = false;

  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance!.addObserver(this);
    initOptions();
  }

  void initOptions() {
    currentChannel ??= _flCameraChannel;
    cameraEven = FLCameraEvent.instance;
    cameraEven.setMethodChannel(currentChannel!);
    cameraMethodCall = FlCameraMethodCall.instance;
    cameraMethodCall.setMethodChannel(currentChannel!);
    WidgetsBinding.instance!.addPostFrameCallback((Duration duration) async {
      await SystemChrome.setPreferredOrientations(
          <DeviceOrientation>[DeviceOrientation.portraitUp]);
      SystemChrome.setSystemUIOverlayStyle(SystemUiOverlayStyle.light
          .copyWith(statusBarColor: Colors.transparent));
    });
  }

  /// 初始化消息通道
  Future<bool> initEvent(EventListen eventListen) async {
    final bool state = await cameraEven.initialize();
    if (state)
      cameraEven.addListener((dynamic data) => _eventListen(eventListen, data));
    return state;
  }

  void _eventListen(EventListen? eventListen, dynamic data) {
    if (data is Map) {
      final bool? state = data['flash'] as bool?;
      FlashState flashState = FlashState.auto;
      if (state != null) flashState = state ? FlashState.on : FlashState.off;
      onFlashChange(flashState);
    } else {
      if (eventListen != null) eventListen(data);
    }
  }

  void onFlashChange(FlashState state) {}

  /// 初始化相机
  Future<bool> initCamera() async {
    bool state = await cameraMethodCall.initCamera();
    if (state) {
      final FlCameraOptions? previewInfo =
          await cameraMethodCall.startPreview();
      state = previewInfo != null;
      if (state) {
        cameraOptions = previewInfo;
        return true;
      }
    }
    return state;
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state != AppLifecycleState.resumed) cameraMethodCall.dispose();
  }

  @override
  Widget build(BuildContext context) {
    Widget current = const SizedBox();
    if (cameraOptions != null && cameraOptions!.textureId != null) {
      final Size size = Size(cameraOptions!.width!, cameraOptions!.height!);

      current = Container(
          color: Colors.black,
          alignment: Alignment.center,
          child: AspectRatio(
              aspectRatio: size.aspectRatio,
              child: Texture(textureId: cameraOptions!.textureId!)));

      if (fullScreen)
        current = ClipRect(
            child: Transform.scale(scale: _fill(size), child: current));
    } else {
      current = uninitialized ??
          const Align(alignment: Alignment.center, child: Text('相机未初始化'));
    }
    return SizedBox.expand(child: current);
  }

  double _fill(Size size) {
    final Size deviceSize = MediaQuery.of(context).size;
    if (deviceSize.aspectRatio < size.aspectRatio) {
      return deviceSize.height * size.aspectRatio / deviceSize.width;
    } else {
      return deviceSize.width / size.aspectRatio / deviceSize.height;
    }
  }

  @override
  void dispose() {
    cameraEven.dispose();
    cameraMethodCall.dispose();
    super.dispose();
    WidgetsBinding.instance!.removeObserver(this);
  }
}

enum FlashState {
  on,
  off,

  /// only ios
  auto
}
